'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var IntlMessageFormat = _interopDefault(require('intl-messageformat'));
var router = require('next/router');

var NextIntlContext = /*#__PURE__*/React.createContext(undefined);

function NextIntlProvider(_ref) {
  var children = _ref.children,
      locale = _ref.locale,
      messages = _ref.messages;
  return React__default.createElement(NextIntlContext.Provider, {
    value: {
      messages: messages,
      locale: locale
    }
  }, children);
}

function useIntlContext() {
  var context = React.useContext(NextIntlContext);

  if (!context) {
    {
      throw new Error('No next-intl context found, please use `NextIntlProvider`.');
    }
  }

  return context;
}

function useLocale() {
  var context = useIntlContext();
  var nextLocale = router.useRouter().locale;
  var locale = context.locale || nextLocale;

  if (!locale) {
    {
      throw new Error("Couldn't determine locale. Please make sure you use internationalized routing or alternatively pass an explicit locale to `NextIntlProvider`.");
    }
  }

  return locale;
}

function resolvePath(messages, idPath) {
  var message = messages;
  idPath.split('.').forEach(function (part) {
    var next = message[part];

    {
      if (part == null || next == null) {
        throw new Error("Could not resolve `" + idPath + "` in `" + JSON.stringify(messages, null, 2) + "`.");
      }
    }

    message = next;
  });
  return message;
}

function prepareTranslationValues(values) {
  if (!values) return values; // Workaround for https://github.com/formatjs/formatjs/issues/1467

  var transformedValues = {};
  Object.keys(values).forEach(function (key) {
    var value = values[key];
    var transformed;

    if (typeof value === 'function') {
      transformed = function transformed(children) {
        var result = value(children);
        return React.isValidElement(result) ? React.cloneElement(result, {
          key: result.key || key + String(children)
        }) : result;
      };
    } else {
      transformed = value;
    }

    transformedValues[key] = transformed;
  });
  return transformedValues;
}
/**
 * Translates messages from the given path by using the ICU syntax.
 * See https://formatjs.io/docs/core-concepts/icu-syntax.
 *
 * If no path is provided, all available messages are returned.
 *
 * The path can also indicate nesting by using a dot (e.g. `namespace.Component`).
 */


function useTranslations(path) {
  var context = useIntlContext();

  {
    if (!context) {
      throw new Error('No `next-intl` context found. Have you configured `NextIntlProvider`?');
    }
  }

  var locale = useLocale();
  var cachedFormatsByLocaleRef = React.useRef({});
  var allMessages = context.messages;

  {
    if (!allMessages) {
      throw new Error('No messages found. Have you configured `NextIntlProvider` correctly with messages?');
    }
  }

  var messages = React.useMemo(function () {
    return path ? resolvePath(allMessages, path) : allMessages;
  }, [allMessages, path]);

  {
    if (!messages) {
      throw new Error("No messages for component `" + path + "` found.");
    }
  }

  function translate(
  /** Use a dot to indicate a level of nesting (e.g. `namespace.nestedLabel`). */
  idPath,
  /** Key value pairs for values to interpolate into the message. */
  values,
  /** Provide custom formats for numbers, dates and times. */
  formats) {
    var _cachedFormatsByLocal;

    if (!locale) {
      {
        throw new Error('No `locale` received from `useRouter()`');
      }
    }

    var cachedFormatsByLocale = cachedFormatsByLocaleRef.current;
    var messageFormat;

    if ((_cachedFormatsByLocal = cachedFormatsByLocale[locale]) != null && _cachedFormatsByLocal[idPath]) {
      messageFormat = cachedFormatsByLocale[locale][idPath];
    } else {
      var message = resolvePath(messages, idPath);

      if (typeof message === 'object') {
        {
          throw new Error("Insufficient path specified for `" + idPath + "` in `" + path + "`.");
        }
      }

      messageFormat = new IntlMessageFormat(message, locale, formats);

      if (!cachedFormatsByLocale[locale]) {
        cachedFormatsByLocale[locale] = {};
      }

      cachedFormatsByLocale[locale][idPath] = messageFormat;
    }

    var formattedMessage = messageFormat.format(prepareTranslationValues(values));

    {
      if (formattedMessage === undefined) {
        throw new Error("Unable to format " + path + "." + idPath);
      }
    }

    return formattedMessage;
  }

  return translate;
}

var MINUTE = 60;
var HOUR = MINUTE * 60;
var DAY = HOUR * 24;
var WEEK = DAY * 7;
var MONTH = DAY * (365 / 12); // Approximation

var YEAR = DAY * 365;

function getRelativeTimeFormatConfig(seconds) {
  var absValue = Math.abs(seconds);
  var value, unit; // We have to round the resulting values, as `Intl.RelativeTimeFormat`
  // will include fractions like '2.1 hours ago'.

  if (absValue < MINUTE) {
    unit = 'second';
    value = Math.round(seconds);
  } else if (absValue < HOUR) {
    unit = 'minute';
    value = Math.round(seconds / MINUTE);
  } else if (absValue < DAY) {
    unit = 'hour';
    value = Math.round(seconds / HOUR);
  } else if (absValue < WEEK) {
    unit = 'day';
    value = Math.round(seconds / DAY);
  } else if (absValue < MONTH) {
    unit = 'week';
    value = Math.round(seconds / WEEK);
  } else if (absValue < YEAR) {
    unit = 'month';
    value = Math.round(seconds / MONTH);
  } else {
    unit = 'year';
    value = Math.round(seconds / YEAR);
  }

  return {
    value: value,
    unit: unit
  };
}

function useIntl() {
  var locale = useLocale();

  function formatDateTime(value, options) {
    return new Intl.DateTimeFormat(locale, options).format(value);
  }

  function formatNumber(value, options) {
    return new Intl.NumberFormat(locale, options).format(value);
  }

  function formatRelativeTime(date, now) {
    var dateDate = date instanceof Date ? date : new Date(date);
    var nowDate = now instanceof Date ? now : new Date(now);
    var seconds = (dateDate.getTime() - nowDate.getTime()) / 1000;

    var _getRelativeTimeForma = getRelativeTimeFormatConfig(seconds),
        unit = _getRelativeTimeForma.unit,
        value = _getRelativeTimeForma.value;

    return new Intl.RelativeTimeFormat(locale, {
      numeric: 'auto'
    }).format(value, unit);
  }

  return {
    formatDateTime: formatDateTime,
    formatNumber: formatNumber,
    formatRelativeTime: formatRelativeTime
  };
}

exports.NextIntlProvider = NextIntlProvider;
exports.useIntl = useIntl;
exports.useTranslations = useTranslations;
//# sourceMappingURL=next-intl.cjs.development.js.map
